#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <math.h>  //include libm

#include "Nokia5110/Nokia5110.h"

#define NUMBER_OF_FUNC 3
int func_number;

float function(float x)
{
    if (func_number == 0)
        return (float)(sin(x) * sin(cos(x)));
    if (func_number == 1)
        return (float)(1 - 4 * x - x * x * x / 17) * sin(x * x);
    if (func_number == 2)
        return (float)(x* x * x * x + x * x * x + x * x + x + 1);
    return (float)(x * x + x + 1);
}

void plot_second_order(float first, float last)
{
    float min_val, max_val, dx, dy, value;
    int i, y_val, old_val;
    min_val = function(first);
    max_val = function(first);
    dx = (last - first) / 80.0;
    for (i = 1; i < 80; i++)
    {
        value = function(first + (float)i * dx);
        if (value < min_val)
            min_val = value;
        else if (value > max_val)
            max_val = value;
    }
    dy = (max_val - min_val) / 44.0;
    value = function(first);
    y_val = (int)((value - min_val) / dy + 0.5);

    LCD_set_XY(80, y_val / 8);
    if (y_val % 8 == 0)
        LCD_write_byte(0x1, 1);
    if (y_val % 8 == 1)
        LCD_write_byte(0x2, 1);
    if (y_val % 8 == 2)
        LCD_write_byte(0x4, 1);
    if (y_val % 8 == 3)
        LCD_write_byte(0x8, 1);
    if (y_val % 8 == 4)
        LCD_write_byte(0x10, 1);
    if (y_val % 8 == 5)
        LCD_write_byte(0x20, 1);
    if (y_val % 8 == 6)
        LCD_write_byte(0x40, 1);
    if (y_val % 8 == 7)
        LCD_write_byte(0x80, 1);
    old_val = y_val;
    for (i = 79; i > 0; i--)
    {
        value = function(first + (float)(80 - i) * dx);
        y_val = (int)((value - min_val) / dy + 0.5);
        LCD_set_XY(i, y_val / 8);
        if (y_val % 8 == 0)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0x3, 1);
            else if ((old_val - y_val) == 3)
                LCD_write_byte(0x7, 1);
            else if (old_val - y_val == 4)
                LCD_write_byte(0xf, 1);
            else if (old_val - y_val > 4)
                LCD_write_byte(0x1f, 1);
            else
                LCD_write_byte(0x1, 1);
            if ((y_val - old_val) == 2)
            {
                LCD_set_XY(i, old_val / 8);
                LCD_write_byte(0x80, 1);
            }
            else if ((y_val - old_val) > 2)
            {
                LCD_set_XY(i, old_val / 8);
                LCD_write_byte(0xc0, 1);
            }
        }
        if (y_val % 8 == 1)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0x6, 1);
            else if ((old_val - y_val) == 3)
                LCD_write_byte(0xe, 1);
            else if ((old_val - y_val) == 4)
                LCD_write_byte(0x1e, 1);
            else if ((old_val - y_val) > 4)
                LCD_write_byte(0x3e, 1);
            else if ((y_val - old_val) == 2)
                LCD_write_byte(0x3, 1);
            else if ((y_val - old_val) == 3)
            {
                LCD_write_byte(0x3, 1);
                LCD_set_XY(i, y_val / 8 - 1);
                LCD_write_byte(0x80, 1);
            }
            else if ((y_val - old_val) == 4)
            {
                LCD_write_byte(0x3, 1);
                LCD_set_XY(i, y_val / 8 - 1);
                LCD_write_byte(0xc0, 1);
            }
            else if ((y_val - old_val) > 4)
            {
                LCD_write_byte(0x3, 1);
                LCD_set_XY(i, y_val / 8 - 1);
                LCD_write_byte(0xe0, 1);
            }
            else
                LCD_write_byte(0x2, 1);
        }
        if (y_val % 8 == 2)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0xc, 1);
            else if ((old_val - y_val) == 3)
                LCD_write_byte(0x1c, 1);
            else if ((old_val - y_val) == 4)
                LCD_write_byte(0x3c, 1);
            else if ((old_val - y_val) > 4)
                LCD_write_byte(0x7c, 1);
            else if ((y_val - old_val) == 2)
                LCD_write_byte(0x6, 1);
            else if ((y_val - old_val) == 3)
                LCD_write_byte(0x7, 1);
            else if ((y_val - old_val) == 4)
            {
                LCD_write_byte(0x7, 1);
                LCD_set_XY(i, y_val / 8 - 1);
                LCD_write_byte(0x80, 1);
            }
            else if ((y_val - old_val) > 4)
            {
                LCD_write_byte(0x7, 1);
                LCD_set_XY(i, y_val / 8 - 1);
                LCD_write_byte(0xc0, 1);
            }
            else
                LCD_write_byte(0x4, 1);
        }
        if (y_val % 8 == 3)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0x18, 1);
            else if ((old_val - y_val) == 3)
                LCD_write_byte(0x38, 1);
            else if ((old_val - y_val) == 4)
                LCD_write_byte(0x78, 1);
            else if ((old_val - y_val) > 4)
                LCD_write_byte(0xf8, 1);
            else if ((y_val - old_val) == 2)
                LCD_write_byte(0xc, 1);
            else if ((y_val - old_val) == 3)
                LCD_write_byte(0xe, 1);
            else if ((y_val - old_val) == 4)
                LCD_write_byte(0xf, 1);
            else if ((y_val - old_val) > 4)
            {
                LCD_write_byte(0xf, 1);
                LCD_set_XY(i, y_val / 8 - 1);
                LCD_write_byte(0x80, 1);
            }
            else
                LCD_write_byte(0x8, 1);
        }
        if (y_val % 8 == 4)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0x30, 1);
            else if ((old_val - y_val) == 3)
                LCD_write_byte(0x70, 1);
            else if ((old_val - y_val) == 4)
                LCD_write_byte(0xf0, 1);
            else if ((old_val - y_val) > 4)
            {
                LCD_write_byte(0xf0, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x1, 1);
            }
            else if ((y_val - old_val)  == 2)
                LCD_write_byte(0x18, 1);
            else if ((y_val - old_val) == 3)
                LCD_write_byte(0x1c, 1);
            else if ((y_val - old_val) == 4)
                LCD_write_byte(0x1e, 1);
            else if ((y_val - old_val) > 4)
                LCD_write_byte(0x1f, 1);
            else
                LCD_write_byte(0x10, 1);
        }
        if (y_val % 8 == 5)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0x60, 1);
            else if ((old_val - y_val) == 3)
                LCD_write_byte(0xe0, 1);
            else if ((old_val - y_val) == 4)
            {
                LCD_write_byte(0xe0, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x1, 1);
            }
            else if ((old_val - y_val) > 4)
            {
                LCD_write_byte(0xe0, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x3, 1);
            }
            else if ((y_val - old_val) == 2)
                LCD_write_byte(0x30, 1);
            else if ((y_val - old_val) == 3)
                LCD_write_byte(0x38, 1);
            else if ((y_val - old_val) == 4)
                LCD_write_byte(0x3c, 1);
            else if ((y_val - old_val) > 4)
                LCD_write_byte(0x3e, 1);
            else
                LCD_write_byte(0x20, 1);
        }
        if (y_val % 8 == 6)
        {
            if ((old_val - y_val) == 2)
                LCD_write_byte(0xc0, 1);
            else if ((old_val - y_val) == 3)
            {
                LCD_write_byte(0xc0, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x1, 1);
            }
            else if ((old_val - y_val) == 4)
            {
                LCD_write_byte(0xc0, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x3, 1);
            }
            else if ((old_val - y_val) > 4)
            {
                LCD_write_byte(0xc0, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x7, 1);
            }
            else if ((y_val - old_val) == 2)
                LCD_write_byte(0x60, 1);
            else if ((y_val - old_val) == 3)
                LCD_write_byte(0x70, 1);
            else if ((y_val - old_val) == 4)
                LCD_write_byte(0x78, 1);
            else if ((y_val - old_val) > 4)
                LCD_write_byte(0x7c, 1);
            else
                LCD_write_byte(0x40, 1);
        }
        if (y_val % 8 == 7)
        {
            if ((old_val - y_val) == 2)
            {
                LCD_write_byte(0x80, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x1, 1);
            }
            else if ((old_val - y_val) == 3)
            {
                LCD_write_byte(0x80, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x3, 1);
            }
            else if ((old_val - y_val) == 4)
            {
                LCD_write_byte(0x80, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0x7, 1);
            }
            else if ((old_val - y_val) > 4)
            {
                LCD_write_byte(0x80, 1);
                LCD_set_XY(i, y_val / 8 + 1);
                LCD_write_byte(0xf, 1);
            }
            else if ((y_val - old_val) == 2)
                LCD_write_byte(0xc0, 1);
            else if ((y_val - old_val) == 3)
                LCD_write_byte(0xe0, 1);
            else if ((y_val - old_val) == 4)
                LCD_write_byte(0xf0, 1);
            else if ((y_val - old_val) > 4)
                LCD_write_byte(0xf8, 1);
            else
                LCD_write_byte(0x80, 1);
        }
        old_val = y_val;
    }
}


int main(void) 
{
	_delay_ms(50);
	LCD_init();       //LCD initialization
	LCD_clear();      //clear the LCD

	DDRA &= ~(1 << PA0) & ~(1 << PA1) & ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4);
	PORTA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) ;
    func_number = 0;
	while(1) 
	{
		
        if (func_number == 0)
            plot_second_order(-4.0, 4.0);
        if (func_number == 1)
            plot_second_order(-2.0, 2.0);
        if (func_number == 2)
            plot_second_order(-1.5, 1.5);
		_delay_ms(2000);
        func_number = (func_number + 1) % NUMBER_OF_FUNC;
        LCD_clear();
	}
}
